{% load taxsystem %}

{% taxsystem_static 'js/payments.js' %}
{% taxsystem_static 'js/modals/payments-reject.js' %}
{% taxsystem_static 'js/modals/payments-approve.js' %}
{% taxsystem_static 'js/modals/payments-undo.js' %}

{% taxsystem_static 'css/cards.css' %}
